@@ -1,38 +1,4 @@
 # -*- coding: utf-8 -*-
-import copy
-import logging
-import os
-import re
-
-import six
-
-import reportlab
-import xhtml2pdf.default
-import xhtml2pdf.parser
-from reportlab.lib.enums import TA_LEFT
-from reportlab.lib.fonts import addMapping
-from reportlab.lib.pagesizes import A4
-from reportlab.lib.styles import ParagraphStyle
-from reportlab.pdfbase import pdfmetrics
-from reportlab.pdfbase.ttfonts import TTFont
-from reportlab.platypus.frames import Frame, ShowBoundaryValue
-from reportlab.platypus.paraparser import ParaFrag, ps2tt, tt2ps
-from xhtml2pdf.util import (copy_attrs, getColor, getCoords, getFile,
-                            getFrameDimensions, getSize, pisaFileObject,
-                            set_value, set_asian_fonts, arabic_format, frag_text_language_check)
-
-from xhtml2pdf.w3c import css
-from xhtml2pdf.xhtml2pdf_reportlab import (PmlPageCount, PmlPageTemplate,
-                                           PmlParagraph, PmlParagraphAndImage,
-                                           PmlTableOfContents)
-
-TupleType = tuple
-ListType = list
-basestring = six.text_type
-try:
-    import urlparse
-except ImportError:
-    import urllib.parse as urlparse
 
 # Copyright 2010 Dirk Holtwick, holtwick.it
 #
@@ -48,8 +14,37 @@ except ImportError:
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-reportlab.rl_config.warnOnMissingFontGlyphs = 0
+import copy
+import logging
+import os
+import re
+from pathlib import Path
 
+from reportlab import rl_settings
+from reportlab.lib.enums import TA_LEFT
+from reportlab.lib.fonts import addMapping
+from reportlab.lib.pagesizes import A4
+from reportlab.lib.styles import ParagraphStyle
+from reportlab.pdfbase import pdfmetrics
+from reportlab.pdfbase.ttfonts import TTFont
+from reportlab.platypus.frames import Frame, ShowBoundaryValue
+from reportlab.platypus.paraparser import ParaFrag, ps2tt, tt2ps
+
+import xhtml2pdf.default
+import xhtml2pdf.parser
+from xhtml2pdf.files import  getFile, pisaFileObject
+from xhtml2pdf.util import (arabic_format, copy_attrs, frag_text_language_check, getColor, getCoords,
+                            getFrameDimensions, getSize, set_asian_fonts, set_value)
+from xhtml2pdf.w3c import css
+from xhtml2pdf.xhtml2pdf_reportlab import (PmlPageCount, PmlPageTemplate, PmlParagraph,
+                                           PmlParagraphAndImage, PmlTableOfContents)
+import urllib.parse as urlparse
+TupleType = tuple
+ListType = list
+
+
+
+rl_settings.warnOnMissingFontGlyphs = 0
 log = logging.getLogger("xhtml2pdf")
 
 sizeDelta = 2       # amount to reduce font size by for super and sub script
@@ -96,7 +91,7 @@ def getParaFrag(style):
               )
     set_value(frag,
               ('pageNumber', 'pageCount', 'outline',
-               'outlineOpen', 'keepWithNext'),
+               'outlineOpen', 'keepWithNext', 'rtl'),
               False)
 
     frag.text = ""
@@ -136,7 +131,7 @@ def getDirName(path):
     if parts.scheme:
         return path
     else:
-        return os.path.dirname(os.path.abspath(path))
+        return str(Path(path).parent.resolve())
 
 
 class pisaCSSBuilder(css.CSSBuilder):
@@ -170,18 +165,16 @@ class pisaCSSBuilder(css.CSSBuilder):
 
         if isinstance(data['src'], list):
             for part in uri:
-                if isinstance(part, basestring):
+                if isinstance(part, str):
                     fonts.append(part)
         else:
             fonts.append(uri)
 
         for font in fonts:
             src = self.c.getFile(font, relative=self.c.cssParser.rootPath)
-            self.c.loadFont(
-                names,
-                src,
-                bold=bold,
-                italic=italic)
+            if not src.notFound():
+                self.c.loadFont(names, src,
+                                bold=bold, italic=italic)
         return {}, {}
 
     def _pisaAddFrame(self, name, data, first=False, border=None, size=(0, 0)):
@@ -415,7 +408,7 @@ class pisaContext(object):
         self.pathDocument = path or "__dummy__"
         parts = urlparse.urlparse(self.pathDocument)
         if not parts.scheme:
-            self.pathDocument = os.path.abspath(self.pathDocument)
+            self.pathDocument = str(Path(self.pathDocument).absolute().resolve())
         self.pathDirectory = getDirName(self.pathDocument)
 
         self.meta = dict(
@@ -527,7 +520,7 @@ class pisaContext(object):
 
     def addTOC(self):
         styles = []
-        for i in six.moves.range(20):
+        for i in range(20):
             self.node.attributes["class"] = "pdftoclevel%d" % i
             self.cssAttr = xhtml2pdf.parser.CSSCollect(self.node, self)
             xhtml2pdf.parser.CSS2Frag(self, {
@@ -602,6 +595,8 @@ class pisaContext(object):
                     language = self.__getattribute__('language')
                     detect_language_result = arabic_format(self.text, language)
                     if detect_language_result != None:
+                        if self.text != detect_language_result:
+                            first.rtl = True
                         self.text = detect_language_result
                 para = PmlParagraph(
                     self.text,
@@ -614,6 +609,7 @@ class pisaContext(object):
                 para.outlineOpen = first.outlineOpen
                 para.keepWithNext = first.keepWithNext
                 para.autoLeading = "max"
+                para.rtl = first.rtl
 
                 if self.image:
                     para = PmlParagraphAndImage(
@@ -763,36 +759,13 @@ class pisaContext(object):
         except:
             return self.context(msg)
 
-    # UTILS
-    def _getFileDeprecated(self, name, relative):
-        try:
-            path = relative or self.pathDirectory
-            if name.startswith("data:"):
-                return name
-            if self.pathCallback is not None:
-                nv = self.pathCallback(name, relative)
-            else:
-                if path is None:
-                    log.warning(
-                        "Could not find main directory for getting filename. Use CWD")
-                    path = os.getcwd()
-                nv = os.path.normpath(os.path.join(path, name))
-                if not (nv and os.path.isfile(nv)):
-                    nv = None
-            if nv is None:
-                log.warning(self.warning("File '%s' does not exist", name))
-            return nv
-        except:
-            log.warning(
-                self.warning("getFile %r %r %r", name, relative, path), exc_info=1)
-
     def getFile(self, name, relative=None):
         """
         Returns a file name or None
         """
-        if self.pathCallback is not None:
-            return getFile(self._getFileDeprecated(name, relative))
-        return getFile(name, relative or self.pathDirectory)
+        if name is None:
+            return
+        return getFile(name, relative or self.pathDirectory, callback=self.pathCallback)
 
     def getFontName(self, names, default="helvetica"):
         """
@@ -800,12 +773,10 @@ class pisaContext(object):
         """
         # print names, self.fontList
         if type(names) is not ListType:
-            if type(names) not in six.string_types:
-                names = str(names)
+            names = str(names)
             names = names.strip().split(",")
         for name in names:
-            if type(name) not in six.string_types:
-                name = str(name)
+            name = str(name)
             font = name.strip().lower()
             if font in self.asianFontList:
                 font = self.asianFontList.get(font, None)
@@ -820,12 +791,10 @@ class pisaContext(object):
         alias = alias if alias is not None else []
         self.fontList[str(fontname).lower()] = str(fontname)
         for a in alias:
-            if type(fontname) not in six.string_types:
-                fontname = str(fontname)
-            self.fontList[str(a)] = fontname
+            self.fontList[str(a)] = str(fontname)
 
+    # TODO: convert to getFile to support remotes fonts
     def loadFont(self, names, src, encoding="WinAnsiEncoding", bold=0, italic=0):
-
         # XXX Just works for local filenames!
         if names and src:
 
@@ -833,7 +802,7 @@ class pisaContext(object):
             src = file.uri
 
             log.debug("Load font %r", src)
-            if names.startswith("#"):
+            if isinstance(names, str) and names.startswith("#"):
                 names = names.strip('#')
             if type(names) is ListType:
                 fontAlias = names
@@ -879,11 +848,11 @@ class pisaContext(object):
 
                 if suffix == "afm":
                     afm = file.getNamedFile()
-                    tfile = pisaFileObject(baseName + ".pfb")
+                    tfile = pisaFileObject(baseName + ".pfb", basepath=file.basepath)
                     pfb = tfile.getNamedFile()
                 else:
                     pfb = file.getNamedFile()
-                    tfile = pisaFileObject(baseName + ".afm")
+                    tfile = pisaFileObject(baseName + ".afm", basepath=file.basepath)
                     afm = tfile.getNamedFile()
 
                 # determine full font name according to weight and style
